/*******************************************************************************
*
*
* File:         hal_dsi_rx_ctrl.h
* Description:  hal mipi dsi rx path control ͷļ
* Version:      V0.1
* Date:         2021-04-06
* Author:       lzy
 *******************************************************************************/
#ifndef __HAL_DSI_RX_CTRL_H__
#define __HAL_DSI_RX_CTRL_H__

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_dsi_datatype.h"
#include "tau_common.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/
#define RX_DCS_QUEUE_MAX_SIZE 20  /* DCS洢г */

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/
typedef struct hal_dcs_execute_entry_t hal_dcs_execute_entry_t;

typedef struct hal_dcs_packet_t hal_dcs_packet_t;

typedef struct hal_dsi_rx_ctrl_handle_t hal_dsi_rx_ctrl_handle_t;

/* DCS CMD ص, עcus_dcs_entry_table, ƥӦDCS ص*/
typedef bool (*hal_dsi_rx_ctrl_dcs_execute)(hal_dsi_rx_ctrl_handle_t *rx_handle, hal_dcs_packet_t *dcs_packet);

/* AP cmd ص, ҪٻCMD ʱע, ΪNULL ʱDSC ָдָparsecus_dcs_entry_tableص */
typedef bool (*hal_dsi_rx_ctrl_read_entry)(uint8_t data_type, uint8_t dcs_cmd, uint8_t param);

/* AP PPS »ص,ΪPPS ԼPPS picture width/height, ڷֱл, עýӿʱڲPPS */
typedef bool (*hal_dsi_rx_ctrl_pps_entry)(uint8_t *pps, uint8_t size, uint32_t pic_width, uint32_t pic_height);

/**
* @brief hal_rx_dbg_event_e select
*/
typedef enum hal_rx_dbg_event_e
{
    HAL_RX_DBG_FS   = 0,  /* Frame start */
    HAL_RX_DBG_EVENT_MAX
} hal_rx_dbg_event_e;

/* RX debug ص,ڻȡframe start ȹdebug */
typedef void (*hal_dsi_rx_ctrl_dbg_entry)(hal_rx_dbg_event_e event);

/**
* @brief  dsi rx ctrl handle struct
*/
typedef struct hal_dsi_rx_ctrl_handle_t
{
    dsi_base_trans_info_t            base_info;                 /* mipi video תϢ */
    dsi_color_code_e                 rx_color_mode;             /* color mode */
    dsi_lane_nume_e                  rx_lanes;                  /* mipi data lane */
    dsi_video_mode_type_e            rx_nonburst_models;        /* transmission packet sequences */
    dsi_virtual_channel_e            rx_vc;                     /* virtual channel number */
    bool                             compress_en;               /* DSC ѹ־ */
    uint32_t                         rx_hsclk_rate;             /* mipi źlane rate */
    uint8_t                          rx_dsc_pps[DSC_PPS_SIZE];  /* DSC ѹPPS */
    const hal_dcs_execute_entry_t    *cus_dcs_entry_table;      /* DCSб */
    hal_dsi_rx_ctrl_read_entry       rx_dcs_read_entry;         /* Hostָݺ,ΪNULLʱrx_dcs_queueעcmd */
    hal_dsi_rx_ctrl_pps_entry        pps_update_entry;          /* PPS Update ʱص,ڷֱлPPS,ΪNULLʱڲ */
    bool                             used;                      /* handleʹñ־λ */
    uint8_t                          pq_marginal;               /* picture quality,Ϊhal_rx_pq_marginal_type_e */
    bool                             direct_mode;               /* video mode ֱͨģʽ,Ԥ,debugʹ */
    hal_dsi_rx_ctrl_dbg_entry        rx_debug_cb;               /* rx debug ص,ĿǰΪյframe start֮صԤdebug */
    hal_err_handle_level_e           err_handler_level;         /* RXմʱģresetȼ, ȼԽresetģԽ */
    bool                             draw_mode;                 /* ģʽ,debugʹ */
#if defined(ISP_568) || defined(ISP_368)
    uint8_t                          rx_strength;               /* ڵRXźǿȣڿУ׼ģʽλ0~7Ĭ3 */
    hight_performan_mode_e           hight_performan_mode;      /* ģʽȼ,οhight_performan_mode_e */
    bool                             pu_optimize;               /* ŻPUʾЧĬΪfalse;true:ŻPUʾʾЧ,߹;false:ͨPUģʽ,͹ */
#endif
    bool                             video_auto_sync;           /* Video mode Զͬ */
} hal_dsi_rx_ctrl_handle_t;

/**
* @brief  DCS command execute entry
*/
typedef struct hal_dcs_execute_entry_t
{
    uint32_t dcs_command;                      /* DCS command */
    hal_dsi_rx_ctrl_dcs_execute execute_func;  /* command Ӧ */
    bool immediately_func;                     /* ִл:true-жִ,false-DCS첽ִ */
} hal_dcs_execute_entry_t;

/**
* @brief  洢 DCS packet ṹ
*/
typedef struct hal_dcs_packet_t
{
    uint32_t data_type;                        /* data type */
    uint32_t dcs_command;                      /* dcs command */
    uint32_t param_length;                     /* dcs param length */
    uint8_t *packet_param;                     /* dcs param */
    const hal_dcs_execute_entry_t *dcs_execute_entry;  /* dcs packet */
} hal_dcs_packet_t;

/**
* @brief dcs command filter select
*/
typedef enum
{
    HAL_RX_DCS_FILTER_0 = 0,
    HAL_RX_DCS_FILTER_1 = 1,
    HAL_RX_DCS_FILTER_2 = 2,
    HAL_RX_DCS_FILTER_3 = 3,
    HAL_RX_DCS_FILTER_4 = 4,
    HAL_RX_DCS_FILTER_5 = 5,
    HAL_RX_DCS_FILTER_6 = 6,
    HAL_RX_DCS_FILTER_7 = 7,
    HAL_RX_DCS_FILTER_MAX
} hal_rx_dcs_filter_sel_e;

/**
* @brief  pentile source color format
*/
typedef enum
{
    PENTILE_SRC_FORMAT_RGB           = 0x0,
    PENTILE_SRC_FORMAT_BGR           = 0x1,
    PENTILE_SRC_FORMAT_RGBG_BGRG     = 0x8,
    PENTILE_SRC_FORMAT_GBGR_GRGB     = 0x9,
    PENTILE_SRC_FORMAT_BGRG_RGBG     = 0xA,
    PENTILE_SRC_FORMAT_GRGB_GBGR     = 0xB,
    PENTILE_SRC_FORMAT_RGBG_RGBG     = 0xC,
    PENTILE_SRC_FORMAT_GBGR_GBGR     = 0xD,
    PENTILE_SRC_FORMAT_BGRG_BGRG     = 0xE,
    PENTILE_SRC_FORMAT_GRGB_GRGB     = 0xF,
    PENTILE_SRC_FORMAT_MAX
} pentile_src_format_e;

/**
* @brief pential G0 G1 swap mode
*/
typedef enum
{
    PENTILE_G0G1  = 0,
    PENTILE_G1G0  = 1
} pentile_g_swap_e;

/**
* @brief pential R B swap mode
*/
typedef enum
{
    PENTILE_RGBG_BGRG  = 0,
    PENTILE_GGRB_RBGG  = 1,
    PENTILE_GGBR_BRGG  = 3
} pentile_rb_swap_e;

/**
* @brief TE źŲģʽ
*/
typedef enum
{
    TE_HW_MODE          = 0,   /* TEӲ,Ƶ֡һ */
    TE_USER_MODE        = 1,   /* ײ㲻TE, hal_dsi_rx_ctrl_gen_a_tear_signal ӿڲ */
    TE_SOFT_60HZ_MODE   = 2,   /* ײͬ60Hz TE   */
    TE_SOFT_90HZ_MODE   = 4,   /* ײͬ90Hz TE  */
    TE_SOFT_120HZ_MODE  = 5,   /* ײͬ120Hz TE   */
    TE_HW_MAX
} te_mode_e;

/**
* @brief pq_marginal_type select
*/
typedef enum
{
    PQ_TYPE_0 = 0x0,
    PQ_TYPE_1 = 0x1,
    PQ_TYPE_2 = 0x3,
    PQ_TYPE_3 = 0x2,
    PQ_TYPE_4 = 0xA,
    PQ_TYPE_5 = 0xE,
    PQ_TYPE_6 = 0xC,
    PQ_TYPE_7 = 0x1A,
    PQ_TYPE_8 = 0x18,
    PQ_TYPE_MAX
} hal_rx_pq_marginal_type_e;

/**
* @brief RX CLK
*/
typedef enum
{
    RX_CLK_100M   = 0,
    RX_CLK_150M   = 1,
    RX_CLK_200M   = 2,
    RX_CLK_300M   = 3,
    RX_CLK_MAX
} hal_rx_clk_e;


/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/

/**
* @brief  dsi rx ctrl handle (ͷʱhal_dsi_rx_ctrl_release_handle)
* @param  none
* @retval dsi rx handle
*/
hal_dsi_rx_ctrl_handle_t *hal_dsi_rx_ctrl_create_handle(void);

/**
* @brief  ͷdsi rx ctrl handle
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_release_handle(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  rx ctrl handle  PPS 
* @param  rx_ctrl_handle: dsi rx handle
* @param  pps: pps 
* @param  pps_size: pps 
* @retval true/false
*/
bool hal_dsi_rx_ctrl_pre_init_pps(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint8_t pps[], uint32_t pps_size);

/**
* @brief  ʼdsi rx ģ
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_init(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  dsi rx ģȥʼ
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_deinit(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  dsi rx
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_start(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  dsi rxָ״̬ (debugʹ, rx_ctrl_handleøýӿ)
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_restart(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  ֹͣdsi rx
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_stop(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  ֶRX clk,һRX CLK ɵײԶ,video modeFIFO FULLʹ
* @param  rxbr_clk: rx clk, Ҫhs_lane_rate/8
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_rx_clk(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, hal_rx_clk_e rxbr_clk);

/**
* @brief   MIPI HOSTĶӦ CMD
* @param  rx_ctrl_handle: dsi rx handle
* @param  data_type: data type
* @param  vc:        virtual channel
* @param  cmd_count: ack command ĳ
* @param  ...      : Ҫ͵command(cmd_count һ)
* @retval true/false
*/
bool hal_dsi_rx_ctrl_send_ack_cmd(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, dsi_ack_data_type_e data_type, dsi_virtual_channel_e vc, uint8_t cmd_count, ...);

/**
* @brief  ʹ鷽ʽظ̰,hal_dsi_rx_ctrl_send_ack_cmdһ
* @param  rx_ctrl_handle: dsi rx handle
* @param  data_size: 鳤,̶Ϊ4
* @param  data: ظcmd,ϸ涨:
* data[0]:DI(data type)
* data[1]:data 0
* data[2]:data 1
* data[3]:ڲpkt type,̶̰Ϊ0
* @retval true/false
*/
bool hal_dsi_rx_ctrl_ack_short_cmd(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint8_t data_size, uint8_t data[]);

/**
* @brief  ʹ鷽ʽظ,hal_dsi_rx_ctrl_send_ack_cmdһ
* @param  rx_ctrl_handle: dsi rx handle
* @param  data_size: 鳤,ΪWord Count + header (header̶Ϊ4)
* @param  data: ظcmd,ϸ涨:
* data[0]:DI(data type)
* data[1]:wc 0 (Word Count Ͱλ)
* data[2]:wc 1 (Word Count ߰λ)
* data[3]:ڲpkt type,̶Ϊ1
* data[N]:
* @retval true/false
*/
bool hal_dsi_rx_ctrl_ack_long_cmd(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint8_t data_size, uint8_t data[]);

/**
* @brief  첽DSCӿ,ִcus_dcs_entry_tableӦDCS immediately_funcΪfalseĺ
* @param  rx_ctrl_handle: dsi rx handle
* @retval true - 1DSC , false - DSC 
*/
bool hal_dsi_rx_ctrl_dsc_async_handler(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  ʹӲfilterҪCMD,MCUԴЧCMDռ
* @param  rx_ctrl_handle: dsi rx handle
* @param  filter_number: filter (0-7)
* @param  cmd_start: Ҫcommand codeʼλ
* @param  cmd_end: Ҫcommand codeֹλ
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_hw_cmd_filter(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle,
                                       hal_rx_dcs_filter_sel_e filter_number,
                                       uint32_t cmd_start, uint32_t cmd_end);

/**
* @brief  ͬ,ڵͼ˺
* @param  rx_ctrl_handle: dsi rx handle
* @param  line_num:  ͬк,Χ1 ~ input height
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_cus_sync_line(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t line_num);

/**
* @brief  ʹpatternmipi(ڲ)
* @param  rx_ctrl_handle: dsi rx handle
* @param  pg_orient: pattern (0:Vertical mode ; 1:Horizontal mode)
* @param  enable: /رpattern
* @retval true/false
*/
bool hal_dsi_rx_ctrl_enable_test_pattern(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint8_t pg_orient, bool enable);

/**
* @brief  TEź
* @param  rx_ctrl_handle: dsi rx handle
* @param  inverse_poly: tearźż
* @param  te_width: tearźſ(0-1023)
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_te_waveform(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, bool inverse_poly, uint32_t te_width);

/**
* @brief  ƻscld filter,ͼ
* @param  rx_ctrl_handle: dsi rx handle
* @param  scld_filter_h: ˮƽfilter
* @param  scld_filter_v: ֱfilter
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_cus_scld_filter(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t scld_filter_h[32][2], uint32_t scld_filter_v[32][2]);

/**
* @brief  ȡAP  BTAظsize
* @param  rx_ctrl_handle: dsi rx handle
* @retval ݴС
*/
uint32_t hal_dsi_rx_ctrl_get_max_ret_size(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  ȡAP Compression Mode Command,ĬΪ0,ʹ
* @param  rx_ctrl_handle: dsi rx handle
* @retval AP compressen_en
*/
bool hal_dsi_rx_ctrl_get_compressen_en(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  һTEź
* @param  rx_ctrl_handle: dsi rx handle
* @retval none
*/
bool hal_dsi_rx_ctrl_gen_a_tear_signal(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  ֱлӿ
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_toggle_resolution(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  ģʽ,ͨΪdebugʹ
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_hight_performan_mode(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  TEźΪģʽ
* @param  rx_ctrl_handle: dsi rx handle
* @retval none
*/
bool hal_dsi_rx_ctrl_set_sw_tear_mode(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief  TEźΪӲģʽ
* @param  rx_ctrl_handle: dsi rx handle
* @retval none
*/
bool hal_dsi_rx_ctrl_set_hw_tear_mode(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

/**
* @brief   pentileʽ
* @param  rx_ctrl_handle: dsi rx handle
* @param  src_format: pentile format
* @param  g_swap: swap G0 G1
* @param  rb_swap: swap R B
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_pentile_format(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, pentile_src_format_e src_format, pentile_g_swap_e g_swap, pentile_rb_swap_e rb_swap);

/**
* @brief   RX escape clk
* @param  rx_ctrl_handle: dsi rx handle
* @param  esc_clk: escape clk λHz,10000000ʱCMDΪ10Mhz
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_cus_esc_clk(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t esc_clk);

/**
* @brief  Զ㲢Ӳfilter
* @param  rx_ctrl_handle: dsi rx handle
* @param  enable: /ر Ӳfilter
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_auto_hw_filter(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, bool enable);

/*
* @brief  DCS cmd ͸ģʽ, Tx init ֮Ч
* @param  enable/disable
* @retval true/false
*/
bool hal_dsi_rx_ctrl_set_dcs_direct_mode(bool enable);

/*
* @brief  ֡޸(video mode)
* @param  rx_ctrl_handle: dsi rx handle
* @param  frame_rateframe rate
*/
bool hal_dsi_rx_ctrl_toggle_input_frame_rate(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, dsi_video_frame_rate_e frame_rate);

/**
* @brief  TEģʽչӿ
* @param  line_num:  ͬк,Χ1 ~ input height
ʼ,stepΪ100𲽼С,ֱȫ˺
* @param  te_mode:  te ģʽ,ʹHW mode
* @retval none
*/
bool hal_dsi_rx_ctrl_set_tear_mode_ex(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t line_num, te_mode_e te_mode);

/**
* @brief  ֱлչӿ
* @param  rx_ctrl_handle: dsi rx handle
* @retval true/false
*/
bool hal_dsi_rx_ctrl_toggle_resolution_ex(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle);

#if !defined(ISP_568) && !defined(ISP_368)
    /* ISP_518/ISP_308 ӿ */
    /**
    * @brief  ƻ Channel Gain ,ͼ
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  gain_r: channel gain coefficient for R
    * @param  gain_g: channel gain coefficient for G
    * @param  gain_b: channel gain coefficient for B
    * @retval true/false
    */
    bool hal_dsi_rx_ctrl_set_cus_pq_gain(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, int r_gain, int g_gain, int b_gain);

    /**
    * @brief  ƻenhance for luma,ͼ
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  enhl_str: Enhance Str
    * @param  enhl_edgeslope: Enhance Edge Slope
    * @retval none
    */
    bool hal_dsi_rx_ctrl_set_cus_pq_enh_lum(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t enhl_str, uint32_t enhl_edgeslope);

    /**
    * @brief  ƻfalse color remove for chroma,ͼ
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  desatstr: Ͷȵ Χ:0-4095
    * @param  desatslope: Ͷȵб Χ:0-4095
    * @retval none
    */
    bool hal_dsi_rx_ctrl_set_cus_pq_enh_chr(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t desatstr, uint32_t desatslope);

    /**
    * @brief  ƻfalse color remove for chroma2,ͼ
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  desatmode: Ͷȵģʽ 0-ͱͶ 1-Ͷ
    * @param  fc_final_alpha: Ͷȵ Χ:0 - 255
    * @param  edge_med_slope: Ͷȵ Χ:0 - 4095
    * @retval none
    */
    bool hal_dsi_rx_ctrl_set_cus_pq_enh_chr2(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t desatmode, uint32_t fc_final_alpha, uint32_t edge_med_slope);

#else
    /* ISP_568/ISP_368 ӿ */
    /**
    * @brief  üvideoಿ֣ڲֻͱmipi900x1792,ʵЧΪ828x1792,ڲüұ߸±
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  crop_width: Ҫü
    * @param  crop_height: Ҫü
    * @retval true/false
    */
    bool hal_dsi_rx_ctrl_crop_video(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint32_t crop_width, uint32_t crop_height);

    /*
    * @brief  ʼģʽ,ȫֵ
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  red_data: صR
    * @param  green_data: صG
    * @param  blue_data: صB
    * @retval none
    */
    void hal_dsi_rx_ctrl_draw_mode_init(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, uint8_t red_data, uint8_t green_data, uint8_t blue_data);

    /*
    * @brief  ɫ
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  x: صx 
    * @param  y: صy 
    * @param  red_data: صR
    * @param  green_data: صG
    * @param  blue_data: صB
    * @retval none
    */
    void hal_dsi_rx_ctrl_set_pixel_data(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, int x, int y, uint8_t red_data, uint8_t green_data, uint8_t blue_data);

    /*
    * @brief  ɫ
    * @param  rx_ctrl_handle: dsi rx handle
    * @param  x1,y1: ʼ
    * @param  x2,y2: յ
    * @param  red_data: صR
    * @param  green_data: صG
    * @param  blue_data: صB
    * @retval none
    */
    void hal_dsi_rx_ctrl_set_rect_pixel_data(hal_dsi_rx_ctrl_handle_t *rx_ctrl_handle, int x1, int x2, int y1, int y2, uint8_t red_data, uint8_t green_data, uint8_t blue_data);
#endif

#endif //__HAL_DSI_RX_CTRL_H__
