/*******************************************************************************
*
*
* File:        hal_i2c_slave.h
* Description  i2c hal file
* Version      V0.1
* Date         2021-10-14
* Author       zhanghz
*******************************************************************************/

#ifndef __HAL_I2C_SLAVE_H__
#define __HAL_I2C_SLAVE_H__

#include "tau_device_datatype.h"
#include "tau_common.h"
#include "string.h"

typedef enum
{
    I2C_S_INT_READ = 0,             //  ж
    I2C_S_INT_RX,                   //  ж
    I2C_S_INT_STOP                  // stop ж
} e_i2c_s_int_status;

#if defined(ISP_568) || defined(ISP_368)
typedef enum
{
    I2C_S_0 = 0,
    I2C_S_1,
    I2C_S_MAX
} i2c_s_index_e;
#endif

typedef void (*hal_i2c_s_callback_t)(e_i2c_s_int_status int_status, size_t receive_num);

/**************************************************************************
* @name : hal_i2c_s_init
* @brief : i2c slave ʼ
* @param[in] : slave_addrӻַ
* @param[in] : addr_bitsӻַλ
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_init(uint8_t slave_addr, uint8_t addr_bits);

/**************************************************************************
* @name : hal_i2c_s_dma_write
* @brief : i2c slave dma 
* @param[in] : txBufferbuffer
* @param[in] : data_sizeݸ
* @return : STATUS_SUCCESS DMA ͨһȫ
* @return : ͳҪµú
* @retval :
**************************************************************************/
status_t hal_i2c_s_dma_write(const uint8_t *txBuffer, size_t data_size);

/**************************************************************************
* @name : hal_i2c_s_nonblocking_read
* @brief : i2c slave ׼
* @param[in] : rxBufferbuffer
* @param[in] : data_size
* @return : STATUS_SUCCESS׼գʱͨŲһʼ
* @return : óҪµú
* @retval :
**************************************************************************/
status_t hal_i2c_s_nonblocking_read(uint8_t *rxBuffer, size_t data_size);

/**************************************************************************
* @name : hal_i2c_s_transfer_complate
* @brief : ȡ i2c slave ״̬
* @param[in] :
* @return : trueݷ
* @return : falseݻڷ
* @retval :
**************************************************************************/
bool hal_i2c_s_write_complate(void);

/**************************************************************************
* @name : hal_i2c_s_read_complate
* @brief : ȡ i2c slave ״̬
* @param[in] :
* @return : ݽո
* @retval :
**************************************************************************/
uint8_t hal_i2c_s_read_complate(void);

/**************************************************************************
* @name : hal_i2c_s_read_complate_clear
* @brief :  i2c slave ״̬
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_read_complate_clear(void);

/**************************************************************************
* @name : hal_i2c_s_set_dma_tx_cycle
* @brief :  I2C cycle ģʽ
* @param[in] :
* @return : ENABLEcycleģʽDISABLEcycleģʽ
* @retval :
**************************************************************************/
void hal_i2c_s_set_dma_tx_cycle(bool enable);

/**************************************************************************
* @name : hal_i2c_s_set_transfer
* @brief :  i2c ӻݽ
* @param[in] :hal_tp_transfer_phone_tmpָ
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_set_transfer(hal_i2c_s_callback_t hal_i2c_s_callback_tmp);

/**************************************************************************
* @name : hal_i2c_s_read_data
* @brief :read data
* @param[in] : rx_data: 
* @return : 1: ɹȡ
* @return : 0:  fifo Ϊ
* @retval :
**************************************************************************/
status_t hal_i2c_s_read_data(uint8_t *rx_data);

/**************************************************************************
* @name : hal_i2c_s_write_data
* @brief :write data
* @param[in] : tx_data: ׼͵
* @return : 1: ÷ͳɹ
* @return : 0:  fifo 
* @retval :
**************************************************************************/
status_t hal_i2c_s_write_data(const uint8_t tx_data);

/**************************************************************************
 * @name : hal_i2c_s_rxfifo_notempty
 * @brief : жϵǰ rxfifo Ƿ
 * @param[in] :
 * @return : true: rxfifo 
 * @return : false: rxfifo û
 * @retval :
 **************************************************************************/
bool hal_i2c_s_rxfifo_notempty(void);

/**************************************************************************
* @name : hal_i2c_s_set_high_impedance
* @brief :  I2C ӻIOΪ̬
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_i2c_s_set_high_impedance(void);
/**************************************************************************
* @name : hal_i2c_s_get_tx_byte_num
* @brief : ȡI2Cӻͳɹֽ
* @param[in] :
* @return :ֽ
* @retval :
**************************************************************************/
int hal_i2c_s_get_tx_byte_num(void);
/**************************************************************************
* @name : hal_i2c_s_deinit
* @brief : i2c IPȥʼصʹܡʱӣ
* @param[in] :slave_num ӻ
* @return :
* @retval :
***************************************************************************/
void hal_i2c_s_deinit(void);
#if defined(ISP_568) || defined(ISP_368)
    /**************************************************************************
    * @name : hal_i2c_s_sel
    * @brief : i2c slave ѡ
    * @param[in] : slaverӻ
    * @return :
    * @retval :
    **************************************************************************/
    void hal_i2c_s_sel(i2c_s_index_e slaver);
#endif
#endif /* __HAL_I2C_SLAVE_H__*/

