/*******************************************************************************
* Copyright (C) 2021-2022, All Rights Reserved.
*
* File:        hal_spi_slave.h
* Description  spi slave hal file
* Version      V0.1
* Date         2021-10-23
* Author       lzy
*******************************************************************************/
#ifndef __HAL_SPI_SLAVE_H__
#define __HAL_SPI_SLAVE_H__

/*******************************************************************************
* 1.Included files
*******************************************************************************/
#include "tau_dsi_datatype.h"
#include "tau_common.h"

/*******************************************************************************
* 2.Global constant and macro definitions using #define
*******************************************************************************/

/*******************************************************************************
* 3.Global structures, unions and enumerations using typedef
*******************************************************************************/
/*
Զģʽevent eg:rx_buffer_size=8, host16byte,
յǰ8byteʱSPI_EVENT_RCV_FULL¼,¼,
ɺhostCS,SPI_EVENT_RCV_CS_HIGH¼
*/
typedef enum
{
    SPI_EVENT_RCV_DATA            = 0,    /* ֶģʽ£SPIS ÿһݼ¼ */
    SPI_EVENT_RCV_FULL,                   /* Զģʽ    SPIS ݵbuffer size¼ */
    SPI_EVENT_RCV_CS_HIGH,                /* Զģʽ    SPIS յCS ߵź   */
} hal_spis_event_e;

typedef struct hal_spi_packet_info_t
{
    uint8_t           *rx_buffer;          /* buffer   */
    uint32_t           rx_buffer_size;     /* buffer size   */
    bool               rx_circle;          /* circle mode   */
    const uint8_t     *tx_buffer;          /* buffer   */
    uint32_t           tx_buffer_size;     /* buffer size   */
    bool               tx_circle;          /* circle mode */
    uint32_t           packet_size;        /* packet size */
} hal_spi_packet_info_t;

typedef void (*hal_spi_slave_cb)(hal_spis_event_e event, hal_spi_packet_info_t *packet_info);

/*******************************************************************************
* 4.Global variable extern declarations
*******************************************************************************/

/*******************************************************************************
* 5.Global function prototypes
*******************************************************************************/
/**
* @brief  ʼspi slave ģ
* @param  cpha: λ
* @param  cpol: 
* @param  dma:  ԶģʽDMA enable
* @retval true/false
*/
bool hal_spi_slave_init(uint8_t cpha, uint8_t cpol, bool dma);

/**
* @brief  spi slave ģȥʼ
* @param  none
* @retval true/false
*/
bool hal_spi_slave_deinit(void);

/**
* @brief  spi slave עص
* @param  cbcall back
* @retval true/false
*/
bool hal_spi_slave_register_callback(hal_spi_slave_cb cb);

/**
* @brief  spi slave enable
* @param  none
* @retval true/false
*/
bool hal_spi_slave_enable(void);

/**
* @brief  spi slave disable
* @param  none
* @retval true/false
*/
bool hal_spi_slave_disable(void);

/**
* @brief  spi slave Զbuffer, ײԶݺcallback, bufferΪNULLʱΪԶģʽ
* @param  bufferԶģʽݽbuffer
* @param  size   Զģʽݽbuffer size
* @param  circlecircle modepacket size buffer size ʱoffset 0д(ݲ֧)
* @retval true/false
*/
bool hal_spi_slave_set_auto_rx_buffer(uint8_t *buffer, uint32_t size, bool circle);

/**
* @brief  spi slave Զbuffer
* @param  bufferԶģʽݷbuffer, bufferΪNULLΪлΪԶģʽ
* @param  size   Զģʽݷbuffer size
* @param  circlecircle modeظbuffer
* @retval true/false
*/
bool hal_spi_slave_set_auto_tx_buffer(const uint8_t *buffer, uint32_t size, bool circle);

/**
* @brief  spi slave Զ
* @param  none
* @retval true/false
*/
bool hal_spi_slave_auto_transfer_start(void);

/**
* @brief  spi slave ֹͣԶ(circle mode packetʹ)
* @param  none
* @retval true/false
*/
bool hal_spi_slave_auto_transfer_abort(void);

/**
* @brief  spi slave flush fifo(circle mode packetʹ)
* @param  none
* @retval true/false
*/
bool hal_spi_slave_flush_fifo(void);

/**
* @brief  reset spis tx,spis
* @param  bufferԶģʽݷbuffer
* @param  size   Զģʽݷbuffer size
* @param  circlecircle modeظbuffer
* @retval true/false
*/
bool hal_spi_slave_reset_tx(const uint8_t *buffer, uint32_t size, bool circle);

/**
* @brief  check spi slave busy(CS status)
* @param  none
* @retval true/false
*/
bool hal_spi_slave_busy(void);

/**
* @brief  ȡrx fifo ǿ
* @param  none
* @retval true/false
*/
bool hal_spi_slave_get_rxfifo_notempty(void);

/**
* @brief  ֶģʽ´rx fifo ȡ
* @param  none
* @retval true/false
*/
bool hal_spi_slave_read_data(uint32_t *data);

/**
* @brief  ֶģʽtx fifo д
* @param  none
* @retval true/false
*/
bool hal_spi_slave_write_data(const uint8_t data);

/**************************************************************************
* @name : hal_spi_s_set_high_impedance
* @brief :  SPI ӻIOΪ̬
* @param[in] :
* @return :
* @retval :
**************************************************************************/
void hal_spi_s_set_high_impedance(void);

#endif /* __HAL_SPI_SLAVE_H__*/

